use crate::libvirt_lib_structures::XMLUuid;
use std::fmt::Display;
use std::net::{Ipv4Addr, Ipv6Addr};

#[derive(serde::Serialize, serde::Deserialize, Debug)]
#[serde(rename = "filterref")]
pub struct NetworkFilterRefXML {
    #[serde(rename = "@filter")]
    pub filter: String,
}

#[derive(serde::Serialize, serde::Deserialize, Debug)]
#[serde(rename = "mac")]
pub struct NetworkFilterRuleProtocolMac {
    #[serde(rename = "@srcmacaddr", skip_serializing_if = "Option::is_none")]
    pub srcmacaddr: Option<String>,
    #[serde(rename = "@scmacmask", skip_serializing_if = "Option::is_none")]
    pub srcmacmask: Option<String>,
    #[serde(rename = "@dstmacaddr", skip_serializing_if = "Option::is_none")]
    pub dstmacaddr: Option<String>,
    #[serde(rename = "@dstmacmask", skip_serializing_if = "Option::is_none")]
    pub dstmacmask: Option<String>,
    #[serde(rename = "@comment", skip_serializing_if = "Option::is_none")]
    pub comment: Option<String>,
}

#[derive(serde::Serialize, serde::Deserialize, Debug)]
#[serde(rename = "arp")]
pub struct NetworkFilterRuleProtocolArpXML {
    #[serde(rename = "@srcmacaddr", skip_serializing_if = "Option::is_none")]
    pub srcmacaddr: Option<String>,
    #[serde(rename = "@srcmacmask", skip_serializing_if = "Option::is_none")]
    pub srcmacmask: Option<String>,
    #[serde(rename = "@dstmacaddr", skip_serializing_if = "Option::is_none")]
    pub dstmacaddr: Option<String>,
    #[serde(rename = "@dstmacmask", skip_serializing_if = "Option::is_none")]
    pub dstmacmask: Option<String>,
    #[serde(rename = "@arpsrcipaddr", skip_serializing_if = "Option::is_none")]
    pub arpsrcipaddr: Option<String>,
    #[serde(rename = "@arpsrcipmask", skip_serializing_if = "Option::is_none")]
    pub arpsrcipmask: Option<u8>,
    #[serde(rename = "@arpdstipaddr", skip_serializing_if = "Option::is_none")]
    pub arpdstipaddr: Option<String>,
    #[serde(rename = "@arpdstipmask", skip_serializing_if = "Option::is_none")]
    pub arpdstipmask: Option<u8>,
    #[serde(rename = "@comment", skip_serializing_if = "Option::is_none")]
    pub comment: Option<String>,
}

#[derive(serde::Serialize, serde::Deserialize, Debug)]
#[serde(rename = "ipvx")]
pub struct NetworkFilterRuleProtocolIpvx {
    #[serde(rename = "@srcmacaddr", skip_serializing_if = "Option::is_none")]
    pub srcmacaddr: Option<String>,
    #[serde(rename = "@srcmacmask", skip_serializing_if = "Option::is_none")]
    pub srcmacmask: Option<String>,
    #[serde(rename = "@dstmacaddr", skip_serializing_if = "Option::is_none")]
    pub dstmacaddr: Option<String>,
    #[serde(rename = "@dstmacmask", skip_serializing_if = "Option::is_none")]
    pub dstmacmask: Option<String>,
    #[serde(rename = "@srcipaddr", skip_serializing_if = "Option::is_none")]
    pub srcipaddr: Option<String>,
    #[serde(rename = "@srcipmask", skip_serializing_if = "Option::is_none")]
    pub srcipmask: Option<u8>,
    #[serde(rename = "@dstipaddr", skip_serializing_if = "Option::is_none")]
    pub dstipaddr: Option<String>,
    #[serde(rename = "@dstipmask", skip_serializing_if = "Option::is_none")]
    pub dstipmask: Option<u8>,

    #[serde(rename = "@comment", skip_serializing_if = "Option::is_none")]
    pub comment: Option<String>,
}

#[derive(serde::Serialize, serde::Deserialize, Debug)]
#[serde(rename = "layer4")]
pub struct NetworkFilterRuleProtocolLayer4<IPv> {
    #[serde(rename = "@srcmacaddr", skip_serializing_if = "Option::is_none")]
    pub srcmacaddr: Option<String>,
    #[serde(rename = "@srcipaddr", skip_serializing_if = "Option::is_none")]
    pub srcipaddr: Option<IPv>,
    #[serde(rename = "@srcipmask", skip_serializing_if = "Option::is_none")]
    pub srcipmask: Option<u8>,
    #[serde(rename = "@dstipaddr", skip_serializing_if = "Option::is_none")]
    pub dstipaddr: Option<IPv>,
    #[serde(rename = "@dstipmask", skip_serializing_if = "Option::is_none")]
    pub dstipmask: Option<u8>,
    /// Start of range of source IP address
    #[serde(rename = "@srcipfrom", skip_serializing_if = "Option::is_none")]
    pub srcipfrom: Option<IPv>,
    /// End of range of source IP address
    #[serde(rename = "@srcipto", skip_serializing_if = "Option::is_none")]
    pub srcipto: Option<IPv>,
    /// Start of range of destination IP address
    #[serde(rename = "@dstipfrom", skip_serializing_if = "Option::is_none")]
    pub dstipfrom: Option<IPv>,
    /// End of range of destination IP address
    #[serde(rename = "@dstipto", skip_serializing_if = "Option::is_none")]
    pub dstipto: Option<IPv>,
    #[serde(rename = "@srcportstart", skip_serializing_if = "Option::is_none")]
    pub srcportstart: Option<u16>,
    #[serde(rename = "@srcportend", skip_serializing_if = "Option::is_none")]
    pub srcportend: Option<u16>,
    #[serde(rename = "@dstportstart", skip_serializing_if = "Option::is_none")]
    pub dstportstart: Option<u16>,
    #[serde(rename = "@dstportend", skip_serializing_if = "Option::is_none")]
    pub dstportend: Option<u16>,
    #[serde(rename = "@state", skip_serializing_if = "Option::is_none")]
    pub state: Option<String>,
    #[serde(rename = "@comment", skip_serializing_if = "Option::is_none")]
    pub comment: Option<String>,
}

#[derive(serde::Serialize, serde::Deserialize, Debug)]
#[serde(rename = "all")]
pub struct NetworkFilterRuleProtocolAllXML<IPv> {
    #[serde(rename = "@srcmacaddr", skip_serializing_if = "Option::is_none")]
    pub srcmacaddr: Option<String>,
    #[serde(rename = "@srcipaddr", skip_serializing_if = "Option::is_none")]
    pub srcipaddr: Option<IPv>,
    #[serde(rename = "@srcipmask", skip_serializing_if = "Option::is_none")]
    pub srcipmask: Option<u8>,
    #[serde(rename = "@dstipaddr", skip_serializing_if = "Option::is_none")]
    pub dstipaddr: Option<IPv>,
    #[serde(rename = "@dstipmask", skip_serializing_if = "Option::is_none")]
    pub dstipmask: Option<u8>,
    /// Start of range of source IP address
    #[serde(rename = "@srcipfrom", skip_serializing_if = "Option::is_none")]
    pub srcipfrom: Option<IPv>,
    /// End of range of source IP address
    #[serde(rename = "@srcipto", skip_serializing_if = "Option::is_none")]
    pub srcipto: Option<IPv>,
    /// Start of range of destination IP address
    #[serde(rename = "@dstipfrom", skip_serializing_if = "Option::is_none")]
    pub dstipfrom: Option<IPv>,
    /// End of range of destination IP address
    #[serde(rename = "@dstipto", skip_serializing_if = "Option::is_none")]
    pub dstipto: Option<IPv>,
    #[serde(rename = "@state", skip_serializing_if = "Option::is_none")]
    pub state: Option<String>,
    #[serde(rename = "@comment", skip_serializing_if = "Option::is_none")]
    pub comment: Option<String>,
}

#[derive(serde::Serialize, serde::Deserialize, Debug, Default)]
#[serde(rename = "rule")]
pub struct NetworkFilterRuleXML {
    #[serde(rename = "@action")]
    pub action: String,
    #[serde(rename = "@direction")]
    pub direction: String,
    #[serde(rename = "@priority")]
    pub priority: Option<i32>,

    /// Match mac protocol
    #[serde(default, rename = "mac", skip_serializing_if = "Vec::is_empty")]
    pub mac_selectors: Vec<NetworkFilterRuleProtocolMac>,

    /// Match arp protocol
    #[serde(default, rename = "arp", skip_serializing_if = "Vec::is_empty")]
    pub arp_selectors: Vec<NetworkFilterRuleProtocolArpXML>,

    /// Match rarp protocol
    #[serde(default, rename = "rarp", skip_serializing_if = "Vec::is_empty")]
    pub rarp_selectors: Vec<NetworkFilterRuleProtocolArpXML>,

    /// Match IPv4 protocol
    #[serde(default, rename = "ip", skip_serializing_if = "Vec::is_empty")]
    pub ipv4_selectors: Vec<NetworkFilterRuleProtocolIpvx>,

    /// Match IPv6 protocol
    #[serde(default, rename = "ipv6", skip_serializing_if = "Vec::is_empty")]
    pub ipv6_selectors: Vec<NetworkFilterRuleProtocolIpvx>,

    /// Match TCP protocol
    #[serde(default, rename = "tcp", skip_serializing_if = "Vec::is_empty")]
    pub tcp_selectors: Vec<NetworkFilterRuleProtocolLayer4<Ipv4Addr>>,

    /// Match UDP protocol
    #[serde(default, rename = "udp", skip_serializing_if = "Vec::is_empty")]
    pub udp_selectors: Vec<NetworkFilterRuleProtocolLayer4<Ipv4Addr>>,

    /// Match SCTP protocol
    #[serde(default, rename = "sctp", skip_serializing_if = "Vec::is_empty")]
    pub sctp_selectors: Vec<NetworkFilterRuleProtocolLayer4<Ipv4Addr>>,

    /// Match ICMP protocol
    #[serde(default, rename = "icmp", skip_serializing_if = "Vec::is_empty")]
    pub icmp_selectors: Vec<NetworkFilterRuleProtocolLayer4<Ipv4Addr>>,

    /// Match all protocols
    #[serde(default, rename = "all", skip_serializing_if = "Vec::is_empty")]
    pub all_selectors: Vec<NetworkFilterRuleProtocolAllXML<Ipv4Addr>>,

    /// Match TCP IPv6 protocol
    #[serde(default, rename = "tcp-ipv6", skip_serializing_if = "Vec::is_empty")]
    pub tcp_ipv6_selectors: Vec<NetworkFilterRuleProtocolLayer4<Ipv6Addr>>,

    /// Match UDP IPv6 protocol
    #[serde(default, rename = "udp-ipv6", skip_serializing_if = "Vec::is_empty")]
    pub udp_ipv6_selectors: Vec<NetworkFilterRuleProtocolLayer4<Ipv6Addr>>,

    /// Match SCTP IPv6 protocol
    #[serde(default, rename = "sctp-ipv6", skip_serializing_if = "Vec::is_empty")]
    pub sctp_ipv6_selectors: Vec<NetworkFilterRuleProtocolLayer4<Ipv6Addr>>,

    /// Match ICMP IPv6 protocol
    #[serde(default, rename = "icmpv6", skip_serializing_if = "Vec::is_empty")]
    pub imcp_ipv6_selectors: Vec<NetworkFilterRuleProtocolLayer4<Ipv6Addr>>,

    /// Match all ipv6 protocols
    #[serde(default, rename = "all-ipv6", skip_serializing_if = "Vec::is_empty")]
    pub all_ipv6_selectors: Vec<NetworkFilterRuleProtocolAllXML<Ipv6Addr>>,
}

#[derive(serde::Serialize, serde::Deserialize, Debug)]
#[serde(rename = "filter")]
pub struct NetworkFilterXML {
    #[serde(rename = "@name")]
    pub name: String,
    #[serde(rename = "@chain", skip_serializing_if = "Option::is_none", default)]
    pub chain: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none", rename = "@priority", default)]
    pub priority: Option<i32>,
    #[serde(skip_serializing_if = "Option::is_none")]
    pub uuid: Option<XMLUuid>,
    #[serde(default, rename = "filterref")]
    pub filterrefs: Vec<NetworkFilterRefXML>,
    #[serde(default, rename = "rule", skip_serializing_if = "Vec::is_empty")]
    pub rules: Vec<NetworkFilterRuleXML>,
}

impl NetworkFilterXML {
    pub fn parse_xml<D: Display>(xml: D) -> anyhow::Result<Self> {
        Ok(quick_xml::de::from_str(&xml.to_string())?)
    }

    pub fn into_xml(self) -> anyhow::Result<String> {
        Ok(quick_xml::se::to_string(&self)?)
    }
}
