import { Button } from "@mui/material";
import React from "react";
import { useNavigate, useParams } from "react-router-dom";
import {
  NWFilter,
  NWFilterApi,
  NWFilterIsBuiltin,
  NWFilterURL,
} from "../api/NWFilterApi";
import { AsyncWidget } from "../widgets/AsyncWidget";
import { ConfigImportExportButtons } from "../widgets/ConfigImportExportButtons";
import { VirtWebRouteContainer } from "../widgets/VirtWebRouteContainer";
import { NWFilterDetails } from "../widgets/nwfilter/NWFilterDetails";

export function ViewNWFilterRoute() {
  const { uuid } = useParams();

  const [nwfilter, setNWFilter] = React.useState<NWFilter | undefined>();

  const load = async () => {
    setNWFilter(await NWFilterApi.GetSingle(uuid!));
  };

  return (
    <AsyncWidget
      loadKey={uuid}
      ready={nwfilter !== undefined}
      errMsg="Failed to fetch network filter information!"
      load={load}
      build={() => <ViewNetworkFilterRouteInner nwfilter={nwfilter!} />}
    />
  );
}

function ViewNetworkFilterRouteInner(p: {
  nwfilter: NWFilter;
}): React.ReactElement {
  const navigate = useNavigate();

  return (
    <VirtWebRouteContainer
      label={`Network filter ${p.nwfilter.name}`}
      actions={
        <span style={{ display: "flex", alignItems: "center" }}>
          <ConfigImportExportButtons
            filename={`nwfilter-${p.nwfilter.name}.json`}
            currentConf={p.nwfilter}
          />

          {!NWFilterIsBuiltin(p.nwfilter) && (
            <Button
              variant="contained"
              style={{ marginLeft: "15px" }}
              onClick={() => navigate(NWFilterURL(p.nwfilter, true))}
            >
              Edit
            </Button>
          )}
        </span>
      }
    >
      <NWFilterDetails nwfilter={p.nwfilter} editable={false} />
    </VirtWebRouteContainer>
  );
}
