import { Autocomplete, TextField } from "@mui/material";
import React from "react";
import { useNavigate } from "react-router-dom";
import { NWFilter, NWFilterURL } from "../../api/NWFilterApi";
import { NWFilterItem } from "../nwfilter/NWFilterItem";

export function NWFilterSelectInput(p: {
  editable: boolean;
  label?: string;
  nwfilters: NWFilter[];
  value?: string;
  onChange?: (name?: string) => void;
  canBeNull: boolean;
}): React.ReactElement {
  const navigate = useNavigate();

  const [open, setOpen] = React.useState(false);

  const selectedValue = p.nwfilters.find((o) => o.name === p.value);
  if (!p.editable && !selectedValue) return <></>;

  if (selectedValue)
    return (
      <NWFilterItem
        value={selectedValue}
        onDelete={p.editable ? () => p.onChange?.(undefined) : undefined}
        onClick={
          !p.editable ? () => navigate(NWFilterURL(selectedValue)) : undefined
        }
      />
    );

  return (
    <Autocomplete
      open={open}
      onOpen={() => {
        setOpen(true);
      }}
      onClose={() => {
        setOpen(false);
      }}
      readOnly={!p.editable}
      options={[...(p.canBeNull ? [undefined] : []), ...p.nwfilters]}
      getOptionLabel={(o) => o?.name ?? "Unspecified"}
      value={selectedValue}
      renderInput={(params) => (
        <TextField {...params} variant="standard" label={p.label} />
      )}
      renderOption={(_props, option) => (
        <NWFilterItem
          dense
          onClick={() => {
            p.onChange?.(option?.name);
            setOpen(false);
          }}
          value={option}
        />
      )}
    />
  );
}
