/* eslint-disable react-x/no-array-index-key */
import DeleteIcon from "@mui/icons-material/Delete";
import {
  Button,
  IconButton,
  Paper,
  Table,
  TableBody,
  TableCell,
  TableContainer,
  TableHead,
  TableRow,
  Tooltip,
} from "@mui/material";
import { VMNetInterfaceFilter } from "../../api/VMApi";
import { TextInput } from "./TextInput";

export function VMNetworkFilterParameters(p: {
  editable: boolean;
  filterref: VMNetInterfaceFilter;
  onChange?: () => void;
}): React.ReactElement {
  if (!p.editable && p.filterref.parameters.length === 0) return <></>;

  const addParameter = () => {
    p.filterref.parameters.push({ name: "", value: "" });
    p.onChange?.();
  };

  return (
    <>
      {p.filterref.parameters.length > 0 && (
        <TableContainer component={Paper}>
          <Table size="small" aria-label="nwfilter parameters">
            <TableHead>
              <TableRow>
                <TableCell>Name</TableCell>
                <TableCell>Value</TableCell>
                {p.editable && <TableCell></TableCell>}
              </TableRow>
            </TableHead>
            <TableBody>
              {p.filterref.parameters.map((row, index) => (
                <TableRow
                  key={index}
                  sx={{ "&:last-child td, &:last-child th": { border: 0 } }}
                >
                  <TableCell
                    component="th"
                    scope="row"
                    style={{ padding: "0px 5px" }}
                  >
                    <TextInput
                      editable={p.editable}
                      value={row.name}
                      onValueChange={(v) => {
                        row.name = v ?? "";
                        p.onChange?.();
                      }}
                    />
                  </TableCell>
                  <TableCell scope="row" style={{ padding: "0px 5px" }}>
                    <TextInput
                      editable={p.editable}
                      value={row.value}
                      onValueChange={(v) => {
                        row.value = v ?? "";
                        p.onChange?.();
                      }}
                    />
                  </TableCell>
                  {p.editable && (
                    <TableCell style={{ padding: "0px" }}>
                      <IconButton
                        onClick={() => {
                          p.filterref.parameters.splice(index, 1);
                          p.onChange?.();
                        }}
                      >
                        <Tooltip title="Remove parameter">
                          <DeleteIcon />
                        </Tooltip>
                      </IconButton>
                    </TableCell>
                  )}
                </TableRow>
              ))}
            </TableBody>
          </Table>
        </TableContainer>
      )}

      {p.editable && (
        <Button onClick={addParameter}>Add a filter ref parameter</Button>
      )}
    </>
  );
}
