import { mdiSecurityNetwork } from "@mdi/js";
import Icon from "@mdi/react";
import DeleteIcon from "@mui/icons-material/Delete";

import {
  Avatar,
  IconButton,
  ListItem,
  ListItemAvatar,
  ListItemButton,
  ListItemText,
} from "@mui/material";
import { NWFilter } from "../../api/NWFilterApi";

export function NWFilterItem(p: {
  value?: NWFilter;
  onClick?: () => void;
  dense?: boolean;
  onDelete?: () => void;
}): React.ReactElement {
  const specs = [];
  if (p.value) {
    if (p.value.rules.length === 1) specs.push(`1 rule`);
    else if (p.value.rules.length > 1)
      specs.push(`${p.value.rules.length} rules`);

    if (p.value.join_filters.length === 1) specs.push(`1 joint filter`);
    else if (p.value.join_filters.length > 1)
      specs.push(`${p.value.join_filters.length} joint filters`);

    if (p.value.priority) specs.push(`priority: ${p.value.priority}`);
  }
  const inner = (
    <>
      <ListItemAvatar>
        <Avatar>
          <Icon path={mdiSecurityNetwork} />
        </Avatar>
      </ListItemAvatar>
      <ListItemText
        primary={
          p.value
            ? `${p.value.name} (${p.value.chain?.protocol ?? "unspecified"})`
            : "Unspecified"
        }
        secondary={specs.join(" / ")}
      />
    </>
  );

  if (p.onClick)
    return (
      <ListItemButton onClick={p.onClick} dense={p.dense}>
        {inner}
      </ListItemButton>
    );

  return (
    <ListItem
      secondaryAction={
        p.onDelete ? (
          <IconButton onClick={p.onDelete}>
            <DeleteIcon />
          </IconButton>
        ) : undefined
      }
    >
      {inner}
    </ListItem>
  );
}
